<?php

namespace MR\Zip\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;

class Configuration extends AbstractHelper
{
    const ZIP_PATH = "payment/mr_zip/";
    const MODULE_NAME = "MR_Zip";

    /**
     *
     * @var \Magento\Framework\Module\ModuleListInterface
     */
    private $_moduleList;

    public function __construct(Context $context)
    {
        parent::__construct($context);
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $this->_moduleList = $objectManager->get("Magento\Framework\Module\ModuleListInterface");
        $this->_logger = $objectManager->get("MR\Zip\Logger\ZipLogger");
    }

    public function getPaymentType($storeId = null)
    {
        return 'Purchase';
    }

    public function getModuleVersion()
    {
        if ($this->_moduleList == null)
            return "M2-unknown";
        return "M2-" . $this->_moduleList->getOne(self::MODULE_NAME)['setup_version'];
    }

    public function getZipClientId($storeId = null)
    {
        return $this->_getZipStoreConfig("client_id", $storeId);
    }

    public function getZipClientSecret($storeId = null)
    {
        return $this->_getZipStoreConfig("client_secret", $storeId, true);
    }

    public function getZipApiEndpoint($storeId = null)
    {
        return $this->_getZipStoreConfig("api_endpoint", $storeId);
    }

    public function getZipAuthTokenEndpoint($storeId = null)
    {
        return $this->_getZipStoreConfig("auth_token_endpoint", $storeId);
    }

    public function getZipApiAudience($storeId = null)
    {
        return $this->_getZipStoreConfig("api_audience", $storeId);
    }

    public function getEnabled($storeId = null)
    {
        return filter_var($this->_getZipStoreConfig("active", $storeId), FILTER_VALIDATE_BOOLEAN);
    }

    public function getDebugFlag($storeId = null)
    {
        return filter_var($this->_getZipStoreConfig("debug_flag", $storeId), FILTER_VALIDATE_BOOLEAN);
    }

    public function getMerchantName($storeId = null)
    {
        return $this->_getZipStoreConfig("merchant_name", $storeId);
    }

    private function _getZipStoreConfig($configName, $storeId = null, $isSensitiveData = false)
    {
        $this->_logger->info("Configuration::_getZipStoreConfig storeId argument:" . $storeId);

        $value = $this->scopeConfig->getValue(self::ZIP_PATH . $configName, \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $storeId);

        if (!$isSensitiveData) {
            $this->_logger->info(__METHOD__ . " configName:{$configName} storeId:{$storeId} value:{$value}");
        } else {
            $this->_logger->info(__METHOD__ . " configName:{$configName} storeId:{$storeId} value:*****");
        }
        return $value;
    }
}
